/* pz.c ... an archiver for Palm OS */

#include <Pilot.h>

#include "pz.h"
#include "pz_rcp.h"


/*
 * constants
 */

#ifndef UNPZ
#define PZ_PREFS_CREATOR PZ_CREATOR
#else
#define PZ_PREFS_CREATOR 'UNPZ'
#endif

#define PZ_PREFS_ID 0
#define PZ_OLD_PREFS_VERSION 0
#define PZ_PREFS_VERSION 1

#define PZ_PREFS_DEFAULT_WHERE where_all
#define PZ_PREFS_DEFAULT_KIND kind_all
#define PZ_PREFS_DEFAULT_SORT_BY sort_by_name
#define PZ_PREFS_DEFAULT_INFO0 info0_kind
#define PZ_PREFS_DEFAULT_INFO1 info_type
#define PZ_PREFS_DEFAULT_INFO2 info_creator


/*
 * global variables
 */

struct pz_prefs prefs;


/*
 * start and stop application
 */

static int start_app(void)
{
    SWord ver;
    Word size;
    Boolean new_search;
    DmSearchStateType state;
    UInt card_no;
    LocalID db_id;

    size = sizeof(struct pz_prefs);
    ver = PrefGetAppPreferences(
	    PZ_PREFS_CREATOR, PZ_PREFS_ID, &prefs, &size, true);
    if (ver != PZ_PREFS_VERSION || size != sizeof(struct pz_prefs)) {
#ifndef UNPZ
	if (ver != PZ_OLD_PREFS_VERSION
	 || size != sizeof(struct pz_old_prefs)) {
	    prefs.where = PZ_PREFS_DEFAULT_WHERE;
	    prefs.kind = PZ_PREFS_DEFAULT_KIND;
	    prefs.sort_by = PZ_PREFS_DEFAULT_SORT_BY;
	    prefs.info0 = PZ_PREFS_DEFAULT_INFO0;
	    prefs.info1 = PZ_PREFS_DEFAULT_INFO1;
	    prefs.info2 = PZ_PREFS_DEFAULT_INFO2;
	}
#endif
	prefs.last_db_top = 0;
    }

    if (prefs.last_form_id == ARCH_FORM_ID) {
	for (new_search = true;
	     DmGetNextDatabaseByTypeCreator(new_search, &state,
		    PZ_DB_TYPE, PZ_CREATOR, false, &card_no, &db_id) == noErr;
	     new_search = false) {
	    if (card_no == prefs.last_selection.card_no
	     && db_id == prefs.last_selection.db_id
	     && open_arch_db(card_no, db_id)) {
		FrmGotoForm(ARCH_FORM_ID);
		return noErr;
	    }
	}
    }

    FrmGotoForm(DB_FORM_ID);
    return noErr;
}

static void stop_app(void)
{
    FrmSaveAllForms();
    FrmCloseAllForms();

    PrefSetAppPreferences(PZ_PREFS_CREATOR, PZ_PREFS_ID, PZ_PREFS_VERSION,
	    &prefs, sizeof(struct pz_prefs), true);
}


/*
 * event loop
 */

static Boolean handle_app_event(EventPtr event)
{
    FormPtr form;

    if (event->eType == frmLoadEvent) {
	form = FrmInitForm(event->data.frmLoad.formID);
	FrmSetActiveForm(form);

	switch (event->data.frmLoad.formID) {
	case DB_FORM_ID:
	    FrmSetEventHandler(form, handle_db_form_event);
	    prefs.last_form_id = event->data.frmLoad.formID;
	    break;

	case ARCH_FORM_ID:
	    FrmSetEventHandler(form, handle_arch_form_event);
	    prefs.last_form_id = event->data.frmLoad.formID;
	    break;

	case INFO_FORM_ID:
	    FrmSetEventHandler(form, handle_info_form_event);
	    break;

	case READ_FORM_ID:
	    FrmSetEventHandler(form, handle_read_form_event);
	    break;
	}

	return true;
    }

    return false;
}

static void event_loop(void)
{
    EventType event;
    Word err;

    do {
	EvtGetEvent(&event, evtWaitForever);
	if (SysHandleEvent(&event) == false
	 && MenuHandleEvent(NULL, &event, &err) == false
	 && handle_app_event(&event) == false) {
	    FrmDispatchEvent(&event);
	}
    } while (event.eType != appStopEvent);
}


/*
 * main entry point
 */

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    if (cmd == sysAppLaunchCmdNormalLaunch) {
	if (start_app() == noErr) {
	    event_loop();
	    stop_app();
	}
    }

    return noErr;
}
